<?xml version="1.0"?>

<st-source>
<time-stamp>From VisualWorks®, 7.3.1 of April 20, 2005 on December 4, 2009 at 3:01:08 pm</time-stamp>
<!-- Changes from: NGUBA-Application(Green.55,bbuxton)  to: NGUBA-Application(Green.56,carry) -->


<class>
<name>MetricWriter</name>
<environment>PROVISO.UBA</environment>
<super>PROVISO.UBA.AbstractStreamWriter</super>
<private>false</private>
<indexed-type>none</indexed-type>
<inst-vars>counters </inst-vars>
<class-inst-vars></class-inst-vars>
<imports></imports>
<category>NGUBA-Streams</category>
<attributes>
<package>NGUBA-Application</package>
</attributes>
</class>

<methods>
<class-id>PROVISO.UBA.MetricWriter</class-id> <category>private</category>

<body package="NGUBA-Application" selector="resetCounters">resetCounters

	counters := IdentityDictionary new.</body>

<body package="NGUBA-Application" selector="initialize">initialize

	super initialize.
	self resetCounters.</body>

<body package="NGUBA-Application" selector="isMetricValid:using:">isMetricValid: aMetric using: aMetricCounter 
	
	aMetric resourceId ifNil: 
		[aMetricCounter increaseBadResourceCount.
		self 
			logInvalidMetric: aMetric
			reason: 'Undefined resource id'.
		^false].
	aMetric formulaId ifNil: 
		[aMetricCounter addBadFormulaName: aMetric formulaName.
		self 
			logInvalidMetric: aMetric
			reason: 'Undefined formula'.
		^false].
	aMetric collectorNumber ifNil: 
		[self 
			logInvalidMetric: aMetric
			reason: 'Undefined collector number'.
		^false].
	^true</body>

<body package="NGUBA-Application" selector="counterFor:">counterFor: aStream 
	
	^self counters 
		at: aStream
		ifAbsentPut: [MetricCounter new]</body>
</methods>

<methods>
<class-id>PROVISO.UBA.MetricWriter</class-id> <category>accessing</category>

<body package="NGUBA-Application" selector="counters">counters

	^counters</body>
</methods>

<methods>
<class-id>PROVISO.UBA.MetricWriter</class-id> <category>processing</category>

<body package="NGUBA-Application" selector="handleNormalTuple:">handleNormalTuple: aMetric 
	
	| stream mr counter |
	self makeSureMetricHasProperConnections: aMetric.
	stream := self streamForMetric: aMetric.
	counter := self counterFor: stream.
	(self isMetricValid: aMetric using: counter) 
		ifFalse: 
			[self 
				triggerEvent: #invalidMetric:
				with: aMetric.
			^nil].
	(self filter 
		valueWithThis: self
		with: aMetric) 
			ifFalse: [^self].
	stream := self streamForMetric: aMetric.
	mr := aMetric class metricRecordClass new updateFrom: aMetric.
	self 
		triggerEvent: #newMetricRecord:onCollector:
		with: mr
		with: aMetric collectorNumber.
	self traceOutgoingTuple: mr.
	stream nextPutRecord: mr.
	counter increaseCount.</body>
</methods>

<methods>
<class-id>PROVISO.UBA.MetricWriter</class-id> <category>control</category>

<body package="NGUBA-Application" selector="commit">commit
	
	self fileStreams keysAndValuesDo: 
		[:eachId :eachStreamDict | 
		eachStreamDict keysAndValuesDo: 
			[:aType :eachStream | 
			self 
				triggerEvent: #aboutToCloseMetricStream:collector:dataType:
				withArguments: {eachStream. eachId. aType}.
			eachStream close.
			(self counterFor: eachStream) logCountsUsing: eachStream filename]].
	super commit.
	self resetStreams.
	self resetCounters.</body>
</methods>


<class>
<name>MetricCounter</name>
<environment>PROVISO.UBA</environment>
<super>Core.Object</super>
<private>false</private>
<indexed-type>none</indexed-type>
<inst-vars>count badResourceCount badFormulaNames badFormulaCount </inst-vars>
<class-inst-vars></class-inst-vars>
<imports></imports>
<category>NGUBA-Application</category>
<attributes>
<package>NGUBA-Application</package>
</attributes>
</class>

<methods>
<class-id>PROVISO.UBA.MetricCounter</class-id> <category>public</category>

<body package="NGUBA-Application" selector="increaseBadResourceCount">increaseBadResourceCount

	badResourceCount := badResourceCount + 1</body>

<body package="NGUBA-Application" selector="logCountsUsing:">logCountsUsing: aFilename

	self system 
		logDebugTwo: #METRIC_STREAM_INFO
		template: 
			'Completed writing metric output: &lt;1s&gt;; wrote &lt;2p&gt; metrics; &lt;3p&gt; bad resource IDs; &lt;4p&gt; bad formula names (&lt;5p&gt;)'
		arguments: 
			{aFilename asString.
			self count.
			self badResourceCount.
			self badFormulaCount.
			self badFormulaNames asArray}</body>

<body package="NGUBA-Application" selector="addBadFormulaName:">addBadFormulaName: aString

	badFormulaNames add: aString.
	badFormulaCount := badFormulaCount + 1.</body>

<body package="NGUBA-Application" selector="increaseCount">increaseCount

	count := count + 1</body>
</methods>

<methods>
<class-id>PROVISO.UBA.MetricCounter</class-id> <category>initialize-release</category>

<body package="NGUBA-Application" selector="initialize">initialize
	
	count := 0.
	badResourceCount := 0.
	badFormulaCount := 0.
	badFormulaNames := Set new</body>
</methods>

<methods>
<class-id>PROVISO.UBA.MetricCounter</class-id> <category>accessing</category>

<body package="NGUBA-Application" selector="badFormulaNames">badFormulaNames
	^badFormulaNames</body>

<body package="NGUBA-Application" selector="count">count
	^count</body>

<body package="NGUBA-Application" selector="badFormulaCount">badFormulaCount

	^badFormulaCount</body>

<body package="NGUBA-Application" selector="badResourceCount">badResourceCount
	^badResourceCount</body>
</methods>


<methods>
<class-id>PROVISO.UBA.MetricCounter class</class-id> <category>creation</category>

<body package="NGUBA-Application" selector="new">new

	^self basicNew initialize</body>
</methods>



</st-source>
